#include <xlic/json.hpp>

#include <xtl/util.hpp>
#include <xtl/json.hpp>

namespace xlic {

inline std::string encode(const xlic::mac_address_list & mac_addresses) {
	std::string result("[");
	auto it = mac_addresses.begin();
	if (it != mac_addresses.end()) {
		result += xtl::concat("\"", to_string(*it++), "\"");
	}
	while (it != mac_addresses.end()) {
		result += xtl::concat(",\"", to_string(*it++), "\"");
	}
	result += "]";
	return result;
}

inline std::string escape(const std::string & input) {
	// Really dumb implementation, sue me
	std::string output;
	for (const char c : input) {
		switch (c) {
			case '"': { output += "\\\""; break; }
			case '\\': { output += "\\\\"; break; }
			case '\b': { output += "\\\b"; break; }
			case '\f': { output += "\\\f"; break; }
			case '\n': { output += "\\\n"; break; }
			case '\r': { output += "\\\r"; break; }
			case '\t': { output += "\\\t"; break; }
			default: { output += c; break; }
		}
	}
	return output;
}

inline std::string encode(const xlic::license & license) {
	return xtl::concat(
		"{",
			"\"license_key\":\"", escape(to_string(license.license_key)), "\","
			"\"signature\":\"", escape(to_string(license.signature)), "\""
		"}"
	);
}

inline std::string encode(const xlic::license_list & licenses) {
	std::string result("[");
	auto it = licenses.begin();
	if (it != licenses.end()) {
		result += encode((it++)->second);
	}
	while (it != licenses.end()) {
		result += xtl::concat(",", encode((it++)->second));
	}
	result += "]";
	return result;
}

std::string json_encode(
	const xlic::serial_number & serial_number,
	const xlic::mac_address_list & mac_addresses
) {
	return xtl::concat(
		"{"
			"\"serial_number\":\"", serial_number, "\","
			"\"mac_addresses\":", encode(mac_addresses), ""
		"}"
	);
}

std::string json_encode(
	const xlic::serial_number & serial_number,
	const xlic::mac_address_list & mac_addresses,
	const xlic::license_key & license_key
) {
	return xtl::concat(
		"{"
			"\"serial_number\":\"", serial_number, "\","
			"\"mac_addresses\":", encode(mac_addresses), ","
			"\"license_key\":\"", escape(to_string(license_key)), "\""
		"}"
	);
}

std::string json_encode(
	const xtl::rsa_signature & signature,
	const xlic::license_key & license_key
) {
	return xtl::concat(
		"{"
			"\"signature\":\"", escape(to_string(signature)), "\","
			"\"license_key\":\"", escape(to_string(license_key)), "\""
		"}"
	);
}

std::string json_encode(
	const xlic::serial_number & serial_number,
	const xlic::mac_address_list & mac_addresses,
	const xlic::license_list & licenses
) {
	return xtl::concat(
		"{"
			"\"serial_number\":\"", serial_number, "\","
			"\"mac_addresses\":", encode(mac_addresses), ","
			"\"licenses\":", encode(licenses),
		"}"
	);
}

} // namespace xlic
